@@ -1,5 +1,6 @@
 import http.server
 import logging
+from os.path import abspath
 
 import requests
 import sys
@@ -31,6 +32,8 @@ ERROR_ENDPOINT = "/oauth/errors"
 # URL to redirect user to upon OAuth success
 SUCCESS_ENDPOINT_URL = "https://okpy.org"  # temporary
 
+TOKEN_PATH = ".token"
+
 
 class OKException(Exception):
     """Base exception class for Bacon/OK integration."""
@@ -82,7 +85,7 @@ def _make_token_post(server, data):
     return body
 
 
-def _make_code_post(server, code, redirect_uri):
+def _make_code_post(server, code, redirect_uri="urn:ietf:wg:oauth:2.0:oob"):
     data = {
         "client_id": CLIENT_ID,
         "client_secret": CLIENT_SECRET,
@@ -105,7 +108,7 @@ def _make_refresh_post(server, refresh_token):
     return info["access_token"], int(info["expires_in"]), info["refresh_token"]
 
 
-def _get_code():
+def _get_code(no_browser):
     """ Make the requests to get OK access code """
 
     host_name = REDIRECT_HOST
@@ -125,12 +128,25 @@ def _get_code():
         "scope": OAUTH_SCOPE,
     }
     url = "{}{}?{}".format(OK_SERVER_URL, AUTH_ENDPOINT, urlencode(params))
-    assert webbrowser.open_new(url)
 
     server = OK_SERVER_URL
     code_response = None
     oauth_exception = None
 
+    try:
+        assert not no_browser
+        assert webbrowser.open_new(url)
+    except AssertionError:
+        print("Couldn't open a web browser, performing manual authentication\n")
+        print("Please navigate to https://go.cs61a.org/auth-code to generate a")
+        print("login code. Then, paste that code below to finish logging in!\n")
+        code = input("Paste login code here: ")
+        try:
+            code_response = _make_code_post(server, code)
+            return code_response
+        except OAuthException as e:
+            raise e
+
     class CodeHandler(http.server.BaseHTTPRequestHandler):
         def send_redirect(self, location):
             self.send_response(302)
@@ -242,7 +258,7 @@ class OAuthSession:
         self._dump()
         return True
 
-    def auth(self, force_reauth=False):
+    def auth(self, force_reauth=False, no_browser=False):
         """
         Returns OAuth access token which can be passed to the server
         for identification. If force_reauth is specified then will
@@ -264,20 +280,25 @@ class OAuthSession:
 
         # Perform OAuth
         print("Token is not available, performing OAuth")
-        self.access_token, expires_in, self.refresh_token = _get_code()
+        self.access_token, expires_in, self.refresh_token = _get_code(no_browser)
         return self.access_token
 
 
-def refresh_token():
-    token = OAuthSession().auth()
-    with open(".token", "w+") as f:
+def set_token_path(path):
+    global TOKEN_PATH
+    TOKEN_PATH = abspath(path)
+
+
+def refresh_token(no_browser=False):
+    token = OAuthSession().auth(no_browser=no_browser)
+    with open(TOKEN_PATH, "w+") as f:
         f.write(token)
     return token
 
 
 def get_token():
     try:
-        with open(".token") as f:
+        with open(TOKEN_PATH) as f:
             return f.read()
     except FileNotFoundError:
         return refresh_token()
