@@ -3,10 +3,8 @@ import re
 from collections import defaultdict
 from contextlib import contextmanager
 
-
-def rel_open(path, *args, **kwargs):
-    root = os.path.dirname(os.path.abspath(__file__))
-    return open(os.path.join(root, path), *args, **kwargs)
+from examtool.api.scramble import latex_escape
+from examtool.api.utils import rel_open
 
 
 def generate(exam):
@@ -24,7 +22,7 @@ def generate(exam):
                 write(fr"\q{{{group['points']}}}")
             else:
                 write(r"\item")
-        write(r"{ \bf " + group["name"] + "}")
+        write(r"{ \bf " + latex_escape(group["name"]) + "}")
         write("\n")
         write(group["tex"])
         write(r"\begin{enumerate}[font=\bfseries]")
